@@ -6,7 +6,7 @@ use warnings;
 use Padre::Document ();
 use Padre::Task::Perl6 ();
 
-our $VERSION = '0.40';
+our $VERSION = '0.41';
 our @ISA     = 'Padre::Document';
 
 # max lines to display in a calltip
@@ -18,7 +18,7 @@ my $COLORIZE_TIMEOUT = 1000; # wait n-millisecond before starting the Perl6 colo
 
 # used for coloring by parrot
 my %perl6_colors = (
-	quote_expression => Px::PADRE_BLUE,
+	quote_expression => Padre::Constant::BLUE,
 	parse            => undef,
 	statement_block  => undef,
 	statementlist    => undef,
@@ -31,11 +31,11 @@ my %perl6_colors = (
 	quote_concat => undef,
 	quote_term => undef,
 	quote_literal => undef,
-	post => Px::PADRE_MAGENTA,
+	post => Padre::Constant::MAGENTA,
 	dotty => undef,
 	dottyop => undef,
-	methodop => Px::PADRE_GREEN,
-	name => Px::PADRE_GREEN,
+	methodop => Padre::Constant::GREEN,
+	name => Padre::Constant::GREEN,
 	identifier => undef,
 	term => undef,
 	args => undef,
@@ -43,12 +43,12 @@ my %perl6_colors = (
 	EXPR => undef,
 	statement_control => undef,
 	use_statement => undef,
-	sym => Px::PADRE_RED,
-	'infix:='  => Px::PADRE_GREEN,
-	'infix:+'  => Px::PADRE_GREEN,
-#   'infix:*'  => Px::PADRE_GREEN,
-#	'infix:/'  => Px::PADRE_GREEN,
-	'infix:,'  => Px::PADRE_GREEN,
+	sym => Padre::Constant::RED,
+	'infix:='  => Padre::Constant::GREEN,
+	'infix:+'  => Padre::Constant::GREEN,
+#   'infix:*'  => Padre::Constant::GREEN,
+#	'infix:/'  => Padre::Constant::GREEN,
+	'infix:,'  => Padre::Constant::GREEN,
 	'infix:..' => undef,
 	'prefix:=' => undef,
 	'infix:|' => undef,
@@ -62,9 +62,9 @@ my %perl6_colors = (
 	scoped => undef,
 	variable_declarator => undef,
 	declarator => undef,
-	variable => Px::PADRE_RED, #Px::PADRE_DIM_GRAY,
+	variable => Padre::Constant::RED, #Padre::Constant::DIM_GRAY,
 	integer => undef,
-	number => Px::PADRE_BROWN,
+	number => Padre::Constant::BROWN,
 	circumfix => undef,
 	param_sep => undef,
 	sigil => undef,
@@ -78,13 +78,13 @@ my %perl6_colors = (
 	signature => undef,
 	for_statement => undef,
 	xblock => undef,
-	lambda => Px::PADRE_GREEN,
+	lambda => Padre::Constant::GREEN,
 );
 
-#    'regex_block' => Px::PADRE_BLACK,
-#    'number' => Px::PADRE_ORANGE,
-#    'param_var' => Px::PADRE_CRIMSON,
-#    '_hash' => Px::PADRE_ORANGE,
+#    'regex_block' => Padre::Constant::BLACK,
+#    'number' => Padre::Constant::ORANGE,
+#    'param_var' => Padre::Constant::CRIMSON,
+#    '_hash' => Padre::Constant::ORANGE,
 
 
 sub text_with_one_nl {
@@ -108,7 +108,8 @@ sub colorize {
 	# temporary overlay using the parse tree given by parrot
 	# TODO: let the user select which one to use
 	# TODO: but the parrot parser in the background
-	#my $perl6 = $self->get_perl6;
+	#require Padre::Plugin::Perl6::Util;
+	#my $perl6 = Padre::Plugin::Perl6::Util::get_perl6();
 	#if ($perl6) {
 	#	$self->_parrot_color($perl6);
 	#	return;
@@ -190,28 +191,18 @@ sub _parrot_color {
 	return;
 }
 
-sub get_perl6 {
-	my $self     = shift;
-
-	my $exe_name = $^O eq 'MSWin32' ? 'perl6.exe' : 'perl6';
-	require File::Which;
-	my $perl6 = File::Which::which($exe_name);
-	if (not $perl6) {
-		if (not $ENV{RAKUDO_DIR}) {
-			my $main = Padre->ide->wx->main;
-			$main->error("Either $exe_name needs to be in the PATH or RAKUDO_DIR must point to the directory of the Rakudo checkout.");
-		}
-		$perl6 = File::Spec->catfile($ENV{RAKUDO_DIR}, $exe_name);
-	}
-
-	return $perl6;
-}
-
+# get Perl6 (rakudo) command line for "Run script" F5 Padre menu item
 sub get_command {
 	my $self     = shift;
 
 	my $filename = $self->filename;
-	my $perl6    = $self->get_perl6;
+	require Padre::Plugin::Perl6::Util;
+	my $perl6    = Padre::Plugin::Perl6::Util::get_perl6();
+	
+	if(not $perl6) {
+		my $main = Padre->ide->wx->main;
+		$main->error("Either perl6 needs to be in the PATH or RAKUDO_DIR must point to the directory of the Rakudo checkout.");
+	}
 
 	return qq{"$perl6" "$filename"};
 }
